#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ldap.h>
#include <sys/wait.h>
#include <errno.h>

/*#define SBASE "cn=ludvig,cn=runeberg,dc=example,dc=com"*/
#define SBASE "dc=example,dc=com"

char *host;

/**
 * Returning the resulting number of entries after a search.
 **/
int check_entries_match(LDAP *ld, LDAPMessage *res) {
    int nentries = 0;
    LDAPMessage *msg;
    for ( msg = ldap_first_message( ld, res ); msg != NULL; msg = ldap_next_message( ld, msg ) ) {
        if (ldap_msgtype(msg) == LDAP_RES_SEARCH_ENTRY)
            nentries++;
    }

    return nentries;
}


/**
 * This functions is used by the main process. It adds the specified
 * entry (cn), searches after it and subsequently deletes it. This is
 * repeated 40 times or until the search fails to find the inserted
 * entry (usually before the 10th iteration).
 **/
void add_entry(LDAP *ld, int count) {

    int rc;
    char dn_add[64];
    int i;

    int loop_count = 0;
    LDAPMessage *ares;

    int entries;
    char cn_val[32];
    pid_t curr_pid = getpid();

    char *oc_values[] = { "top", "person", "organizationalPerson", NULL };
    char *tn_values[] = { "2222222", NULL };
    char *sn_values[] = { "donald duck", NULL };
    char *cn_values[] = { cn_val, NULL };
    char *sa_values[] = { "Duck Street", NULL };
    char *pc_values[] = { "12222", NULL };
    char *ra_values[] = { "Goose Street", NULL };
    char *pa_values[] = { "Marabou Street", NULL };
    char *title_values[] = { "Prime Minister", NULL };
    char *desc_values[] = { "The quick brown fox jumped over the lazy dog.", NULL };
    char *pDON_values[] = { "Somewhere in the world.", NULL };

    LDAPMod modattrs[] = { { 0, "objectClass", { oc_values } },
                             { 0, "telephoneNumber", { tn_values } },
                             { 0, "sn", { sn_values } },
                             { 0, "cn", { cn_values } },
                             { 0, "streetAddress", { sa_values } },
                             { 0, "postalCode", { pc_values } },
                             { 0, "registeredAddress", { ra_values } },
                             { 0, "postalAddress", { pa_values } },
                             { 0, "title", { title_values } },
                             { 0, "description", { desc_values } },
                             { 0, "physicalDeliveryOfficeName", { pDON_values} } };
    LDAPMod *attrp[12];
    for (i=0; i<11; i++) {
        attrp[i] = &modattrs[i];
    }
    attrp[11] = NULL;


    while (loop_count < count) {
        entries = 0;
        printf("Loop nr: %d\n", loop_count+1);
        sprintf(cn_val, "dlt%d-%d", curr_pid, loop_count);
        sprintf(dn_add, "cn=%s,dc=example,dc=com", cn_val );
        rc = ldap_add_ext_s(ld, dn_add, attrp, NULL, NULL);
        if ( rc != LDAP_SUCCESS ) {
            printf("add error %d: %s\n", rc, ldap_err2string(rc));
            exit(1);
        }
        
        
        /**
         * Search for it again.
        **/
       ++loop_count;
    }

}

int main( int argc, char ** argv) {
    /**
     * Connect to openldap.
     **/
    char *cn_search = argv[2];
    char *manager = "cn=admin,dc=example,dc=com";
    char url[32];
    LDAP *ld;
    LDAPMessage *res;
    int rc;
    struct berval cred;
    int entries;
    pid_t pID;
    char dn_add[64];
    int cnt;


    int version = LDAP_VERSION3;

    if (argc != 3) {
        fprintf(stderr, "Usage: cacheTestFork <host> <nr of entries>\n");
        exit(-1);
    }

    cnt = atoi(argv[2]);

    host = strdup(argv[1]);

    sprintf(url, "ldap://%s", host);


    
    cred.bv_val = strdup("secret");
    cred.bv_len = sizeof("secret") - 1;
   

    ldap_initialize( &ld, url );

    if ( ld == NULL ) {
        perror( "ldap_init" );
        exit( 1 );
    }

    ldap_set_option( ld, LDAP_OPT_PROTOCOL_VERSION, &version );

    rc = ldap_sasl_bind_s( ld, manager, LDAP_SASL_SIMPLE, &cred, NULL, NULL, NULL);
    while ( rc != LDAP_SUCCESS ) {
        rc = ldap_sasl_bind_s( ld, manager, LDAP_SASL_SIMPLE, &cred, NULL, NULL, NULL);
    }

    /*sprintf(dn_add, "cn=%s,cn=ludvig,cn=runeberg,dc=example,dc=com", cn_search );*/
    sprintf(dn_add, "cn=%s,dc=example,dc=com", cn_search );
    rc = ldap_delete_ext_s(ld, dn_add, NULL, NULL);
    if ( rc != LDAP_SUCCESS ) {
        /* It's ok if the entry doesn't exist. */
        if (rc != 32) {
            printf("delete error %d: %s\n", rc, ldap_err2string(rc));
            exit(1);
        }
    }

    add_entry(ld, cnt);

    return 0;
}
